/******************************************************************************
 *
 *  .----------------------.	
 *  |==     ==  ======  == |	$Author: yujik $
 *  | ==   ==   ======  == |	$Revision: 
 *  |  == ==      ==    == |	$Date: 2009-05-04 16:12:21 -0700 (Mon, 04 May 2009) $ 
 *  |   ===       ==    == |	$HeadURL: http://10.1.0.10/svn/nextgen/trunk/software/Instrument_drivers/libVTEX/common/libInherent.cpp $
 *  |    =        ==    == |	
 *  `----------------------'
 *
 * Copyright 2006 VXI Technology Inc. as an unpublished work.
 * All rights reserved. The information contained herein is confidential
 * property of VXI Technology, Inc. Use, copy, transfer or disclosure
 * of such information is prohibited except by express written agreement
 * with VTI Instruments, Co.
 *
 * PURPOSE:
 *   IVI inherent capabilities Utility interface
 *
 ******************************************************************************/


/*******************
 * System header
 *******************/


/*******************
 * Library header
 *******************/


/*******************
 * Local header
 *******************/
// #include "utilsContainer.h"
#include "functionLogging.h"
#include "iviOptions.h"
#include "errorManager.h"
#include "impInherent.h"
#include "state.h"

/***********************
 * Forward Declarations
 ***********************/


ImpInherent::ImpInherent(UtilsContainer *utils)
{
	if(NULL == utils)
	{
		try
		{
			m_utils = new UtilsContainer();
			m_freeUtils = true;
		}
		catch(std::exception e)
		{
			throw IVI_E_OUT_OF_MEMORY;
		}
	}
	else 
	{
		m_utils = utils;
		m_freeUtils = false;
	}
}   

ImpInherent::~ImpInherent() 
{ 
	if(m_utils && m_freeUtils) delete m_utils;
	m_utils = 0;
}


VTEXResult ImpInherent::Close()
{
    m_utils->log->Log(__FUNCTION__, 0);

    VTEXResult status = m_baseState->initialized();

    if(status >= IVI_SUCCESS)
    {
        if(!m_utils->originallySimulating())
        {
          m_utils->setSimulating(false); //Turn simulating off so we clean up our connection state.
        }
        status = cleanUp();
        m_baseState->setInitialized(false);     //should the driver be listed as uninitialized if an error occurred?
    }

    return m_utils->errorManager->ReportError(status);
}


VTEXResult ImpInherent::get_Initialized(bool *val)
{
    m_utils->log->Log(__FUNCTION__, 0);

    VTEXResult status = IVI_SUCCESS;

    if(NULL == val)
    {
        status = m_utils->errorManager->queueError(IVI_E_NULL_POINTER, __FUNCTION__, "val");
    }
    else
    {
        *val = m_baseState->initialized() == IVI_SUCCESS;
    }

    return m_utils->errorManager->ReportError(status);
}

VTEXResult ImpInherent::Initialize(const VTEXString& resourceName, 
                                   bool idQuery, 
                                   bool reset)
{
    return Initialize(resourceName, idQuery, reset, VTEX_STRING(""));
}


VTEXResult ImpInherent::Initialize(const VTEXString& _resourceName, 
                                   bool idQuery, 
                                   bool reset, 
                                   const VTEXString& _optionString)
{
	std::string resourceName = STD_STRING(_resourceName);
	std::string optionString = STD_STRING(_optionString);

    m_utils->log->Log(__FUNCTION__, 
                      4,
                      LOG(resourceName),
                      LOG(idQuery),
                      LOG(reset),
                      LOG(optionString));

    
    //First, clear the error queue. We always want to do this on Initialize. 
    m_utils->errorManager->clearError();
    VTEXResult status = m_baseState->initialized();
    if(status == IVI_SUCCESS)
    {
        status = IVI_E_ALREADY_INITIALIZED;
    }

    else
    {
        status = IVI_SUCCESS;

		// Important!!
		// Initialized flag nees to be set to true here
		// becasue the "Utils->Reset" call in this function fails unless this flag is set to true.
		m_baseState->setInitialized(true);  //unset this later if this call was not successful
	}

    // An exception can be thrown in the following functions without returning status.
	// So, it must be caught and re-thrown.
	try
	{
        //Parse the option string
        // std::vector<Resource> resourceList;
        if(status >= IVI_SUCCESS)
        {
            status = m_utils->parseOptionString( m_baseState->getPluginType(), 
                                             resourceName, 
                                             optionString, 
                                             m_resourceList,
                                             m_baseState->ignoreMissingSlotsInSimulationMode() );
        }
    if (m_utils->simulating())
    {
      //We are simulating during the Initialize call, so we need to remember to drop out of simulating during Close, before cleanUp happens.
      m_utils->setOriginallySimulating(true);
    }
    else
    {
      m_utils->setOriginallySimulating(false);
    }

        //Init the EXP IO layer
        if(status >= IVI_SUCCESS)
        {
            //status = m_utils->expManager->init(resourceList);
		    status = initEXPManager();
        }

        if(status >= IVI_SUCCESS)
        {
            status = init(reset);
        }  

	}
    catch(VTEXException e)
    {
		status = e.errorCode;
    }
	catch (...)
	{
		// Unknown exception.  Let' assume out of memory.
		status = IVI_E_OUT_OF_MEMORY;
	}

    if(status < IVI_SUCCESS)
    {
        cleanUp();

        m_baseState->setInitialized(false);
    }

    m_utils->iviOptions->bIdQuery = idQuery;
    m_utils->iviOptions->bReset = reset;  


    /*For the initialize call, we don't want to report instrument errors.  So, temporarilly 
     * disable to the QueryInstrumentStatus flag*/
    bool queryInstrumentStatus = m_utils->iviOptions->bQueryInstrStatus;
    m_utils->iviOptions->bQueryInstrStatus = false;
    m_utils->errorManager->ReportError(status);
    m_utils->iviOptions->bQueryInstrStatus = queryInstrumentStatus;

    return m_utils->errorManager->ReportError(status);
}

VTEXResult ImpInherent::initEXPManager()
{
	return m_utils->expManager->init(m_resourceList);
}

#if defined(_WIN32) && defined(DLL_EXPORTS)
#include <comdef.h>

BSTR STDSTR2BSTR(std::string str)
{
	_bstr_t bstr = str.c_str();
	return bstr.Detach();
};

std::string BSTR2STDSTR(BSTR bstr)
{	
	std::string str = _bstr_t (bstr);
	return str;
};

#define COPY2ARRAY(type)	{\
	SAFEARRAY *pArray = NULL;\
	pArray = ::SafeArrayCreateVector(type, (ULONG)0, (ULONG)vect.size());\
	for(unsigned long i=0; i<vect.size(); i++)\
	{\
		::SafeArrayPutElement(pArray, (LONG *)&i, &vect.at(i));\
	}\
	return pArray;\
}




SAFEARRAY *bool_VECT2SAFEARRAY(std::vector<bool> vect)
{
	SAFEARRAY *pArray = NULL;
	pArray = ::SafeArrayCreateVector(VT_BOOL, (ULONG)0, (ULONG)vect.size());
	for(unsigned long i=0; i < vect.size(); i++)
	{
		VARIANT_BOOL bValue = (vect.at(i) != false) ? VARIANT_TRUE : VARIANT_FALSE;
		::SafeArrayPutElement(pArray, (LONG *)&i, &bValue);
	}
	return pArray;
};

SAFEARRAY *int_VECT2SAFEARRAY(std::vector<int> vect)
{
COPY2ARRAY(VT_INT);
};

SAFEARRAY *double_VECT2SAFEARRAY(std::vector<double> vect)
{
COPY2ARRAY(VT_R8);
};

#endif

